CHROMEDRIVER_CHROMIUM_URL = 'https://chromedriver.chromium.org'
CHROMEDRIVER_STORAGE_URL = 'https://chromedriver.storage.googleapis.com'

ZIP_TYPE = '.zip'

CHROMEDRIVER = 'chromedriver'
WEBDRIVER = 'webdriver'
FILE_NAME_CHROMEDRIVER = 'chromedriver'
FILE_NAME_CHROMEDRIVER_EXE = 'chromedriver.exe'

UL_VERSIONS_SELECTOR = '#sites-canvas-main-content > table > tbody > tr > td > div > div:nth-child(5) > ul'

LATEST_STABLE_VERSION_STR = 'Latest stable release'
LATEST_BETA_VERSION_STR = 'Latest beta release'
